"""" Fattura Elettronica Ordinaria. Versione riassuntiva """
from decimal import Decimal

from django.db import models
from django.db.models import Sum
from django.db.models.functions import Coalesce

from minigest.anagrafica.models import SoggettoFiscale

from ..tipo_documento import TipoDocumento
from ...helpers import fatture_dir


class Fattura(models.Model):
    fornitore = models.ForeignKey(
        SoggettoFiscale, on_delete=models.PROTECT, related_name="fatture_vendita"
    )
    cliente = models.ForeignKey(
        SoggettoFiscale, on_delete=models.PROTECT, related_name="fatture_acquisto"
    )

    tipo_documento = models.ForeignKey(TipoDocumento, on_delete=models.PROTECT)

    data = models.DateField()
    data_sdi = models.DateField(
        help_text="Data di ricezione/consegna da parte del Sistema di Interscambio",
    )

    numero = models.CharField(max_length=20)

    art_73 = models.BooleanField(
        null=True,
        blank=True,
        default=None,
        help_text="Indica se il documento è stato emesso secondo modalità e termini stabiliti con decreto ministeriale ai sensi dell'articolo 73 del DPR 633/72 (ciò consente al cedente/prestatore l'emissione nello stesso anno di più documenti aventi stesso numero)",
    )

    importo_totale_documento = models.DecimalField(
        max_digits=19,
        decimal_places=2,
    )

    file = models.FileField(blank=True, default=None, upload_to=fatture_dir)

    @property
    def totale(self):
        return self.importo_totale_documento

    @property
    def imposta(self):
        imposta = self.riepiloghi_iva.all().aggregate(
            totale=Coalesce(Sum("imposta"), Decimal(0))
        )
        return Decimal(format(imposta.get("totale"), ".2f"))

    @property
    def imponibile(self):
        return Decimal(self.totale - self.imposta)

    @property
    def saldo(self):
        if self.distinte.all():
            totale_pagamenti = self.distinte.all().aggregate(
                totale=Coalesce(Sum("importo"), Decimal(0))
            )
            return format(totale_pagamenti.get("totale") - self.totale, ".2f")
        else:
            return 0

    def __str__(self):
        return f"{self.tipo_documento} {self.numero} di {self.fornitore}"

    class Meta:
        verbose_name_plural = "Fatture"
